if ("undefined" == typeof(compromisedstores)) {

	var compromisedstores = { };
	
	function compromisedstores() {
	  
		this.prefManager = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
	  		
		// initialization code
		this.initialized = true;
		this.strings = document.getElementById("compromisedstores-strings");
		
		this.md5 = Components.classes["@mozilla.org/security/hash;1"]
			.createInstance(Components.interfaces.nsICryptoHash);  
		
		this.converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
			.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		this.converter.charset = "UTF-8";
		
		// Database initialization
		this.dbh = this.getDbh();
		if (this.prefManager.getBoolPref("extensions.compromisedstores.firstrun") == true || this.dbh.tableExists("domains") == false) { // first time
			this.dump("First run");
			try {
				this.dbh.executeSimpleSQL("CREATE TABLE domains ('domain' TEXT, 'type' TEXT)");
				setTimeout(function(thisObj) { thisObj.fetchDomains(); }, 5000, this);
			}
			catch(e) {
				this.dump(e);
			}
			finally {
				this.prefManager.setBoolPref("extensions.compromisedstores.firstrun", false);
			}
		}

		this.update_interval = setInterval(function(thisObj) { thisObj.checkUpdate(); }, 60 * 60 * 1000, this); // hourly
	  
		//this.register();
		//this.updateStatus();
	};
	
	compromisedstores.prototype = {	
		onUnload: function() {
			this.unregister();
		},

		dump: function(aMessage) {
			/*var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
			consoleService.logStringMessage("CompromisedStores: " + aMessage + "\n");*/
		},
		
		register: function() {
			this.dump("register");
		
			if ("undefined" == typeof(gBrowser)) { // mobile
				//window.messageManager.loadFrameScript("chrome://compromisedstores/content/mobile.js", true);
				window.messageManager.addMessageListener("Content:LocationChange", compromisedstores.receiveMessage);
				addEventListener("AddonOptionsLoad", compromisedstores.initOptions, false);
			}
			else {
				gBrowser.addTabsProgressListener(this);
				this.updateStatus();
			}
		},
		unregister: function() {
			if ("undefined" == typeof(gBrowser)) { // mobile
				window.messageManager.removeMessageListener("Content:LocationChange", compromisedstores.receiveMessage);
			}
			else {
				gBrowser.removeTabsProgressListener(this);
			}
			
			this.dbh.asyncClose();
		},
		
		toHexString: function (charCode) {
			return ("0" + charCode.toString(16)).slice(-2);
		},
		
		getDbh: function() {
			var dbfile = Components.classes["@mozilla.org/file/directory_service;1"]  
						  .getService(Components.interfaces.nsIProperties)  
						  .get("ProfD", Components.interfaces.nsIFile);  
			dbfile.append("compromisedstores.sqlite");
			var storageService = Components.classes["@mozilla.org/storage/service;1"]  
							 .getService(Components.interfaces.mozIStorageService);  
			var dbh  = storageService.openDatabase(dbfile);

			return dbh;
		},
		
		fetchDomains: function() {
			var req = new XMLHttpRequest(); 
			var url = this.prefManager.getCharPref("extensions.compromisedstores.dataurl");
			this.dump("fetchDomains URL: " + url); 


			var instance = this;
			req.open('GET', url, true);
			req.channel.loadFlags |= Components.interfaces.nsIRequest.LOAD_BYPASS_CACHE; // no cache
			req.onreadystatechange = function (aEvt) {  
				if (req.readyState == 4) {  
					if(req.status == 200)  {
	// 					this.dump(req.responseText);
						var data = JSON.parse(req.responseText); // requires FF 3.5 minimum
						instance.dbh.executeSimpleSQL("DELETE FROM domains");

						try {
							var insert = instance.dbh.createStatement("INSERT INTO domains ('domain', 'type') VALUES (:domain, :type)");
							for(var i in data.domains) {
								insert.params.domain = data.domains[i].domain;
								insert.params.type = data.domains[i].type;

								//instance.dump(data.domains[i].domain);

								try {
									//insert.execute();
									insert.executeAsync();
								}
								catch (e) {
									instance.dump(e);
								}
							}
						}
						catch (e) {
							instance.dump(e);
						}
						finally {
							var currentDate = new Date();
							instance.prefManager.setIntPref("extensions.compromisedstores.lastupdate", currentDate.getTime() / 1000);
						}
					}
					else {
						instance.dump("Error loading JSON data"); 
					}
				}  
			};  
			req.send(null); 
		},
		
		getDomain: function(md5) {
			var statement = compromisedstores.dbh.createStatement("SELECT domain, type FROM domains WHERE domain = :domain");
			statement.params.domain = md5;
			compromisedstores.dump("check domain for " + statement.params.domain);

			while (statement.executeStep()) {  
				var domain = {
					"md5"  : statement.row.domain,
					"type" : statement.row.type,
				};

				return domain;
			} 

			return { "md5" : "", "type" : "" };
		},
		
		initOptions: function(event) {
			if (event.target.id != "urn:mozilla:item:{FC93BE36-E090-11DF-9D98-06A7DFD72085}") {
				return;
			}
		
			var list = document.getElementById("alert-type");
			var item = list.querySelector("[value=\""+ compromisedstores.prefManager.getIntPref("extensions.compromisedstores.alert_type") +"\"]");
			if (item) {
				list.selectedItem = item;
			}
		},
		
		receiveMessage: function(aMessage) {
			/*var consoleService = Cc["@mozilla.org/consoleservice;1"]
				  .getService(Ci.nsIConsoleService);
			consoleService.logStringMessage("received " + aMessage.name);*/
			//compromisedstores.dump("received " + aMessage.name);
			
			var type = '';

			  // determine what sort of message we're dealing with
			if (aMessage.name == "Content:LocationChange") {
				var location = aMessage.json.location;
				
				if (typeof(location.hostname) == "undefined" || location.hostname == null || location.hostname == '') {
					compromisedstores.dump("No location.hostname, skip");
					return;
				}
				
				//compromisedstores.dump("Location: " + location);
				var host = location.hostname;
				type = compromisedstores.domainType(host);
				
				/*if (location.indexOf("http://") == 0) {
					var scheme = "http://";
					var host = location.substr( scheme.length );
					host = host.split("/")[0];
					//compromisedstores.dump("Host: " + host);
					type = compromisedstores.domainType(host);
				}
				else if (location.indexOf("https://") == 0) {
					var scheme = "https://";
					var host = location.substr( scheme.length );
					host = host.split("/")[0];
					//compromisedstores.dump("Host: " + host);
					type = compromisedstores.domainType(host);
				}*/
				
				compromisedstores.dump("Domain type: " + type);
				if (type != '') {
					var alert_type = compromisedstores.prefManager.getIntPref("extensions.compromisedstores.alert_type");
					
					if (alert_type == 1) {
						compromisedstores.showWarningMobileDesktop(type);
					}
					else if (alert_type == 2) {
						compromisedstores.showWarningMobilePanel(type);
					}
					else if (alert_type == 3) {
						compromisedstores.showWarningMobilePopup(type);
					}
				}
			}
		},
		
		domainType: function(host) {
			try {
				var domains = compromisedstores.getDomains(host);
				
				compromisedstores.dump("Number of domains: " + domains.length);
				
				var i = 0;
				while (i < domains.length) {
					var hex = this.md5sum(domains[i]);
					
					compromisedstores.dump(domains[i] + " => " + hex + " - " + i);
					
					// Whitelist
					var whitelist = compromisedstores.prefManager.getCharPref("extensions.compromisedstores.whitelist").split("\n");
					for(var j = 0; j < whitelist.length; j++) {
						//if (compromisedstores.md5sum(whitelist[j]) == hex) {
						if (whitelist[j] == domains[i]) {
							compromisedstores.dump("URL whitelisted: " + domains[i] + " by " + whitelist[j]);
							return ''; // whitelisted, no change
						}
					}
					
					var result = compromisedstores.getDomain(hex);
					if (result.md5 == hex) {
						compromisedstores.dump(domains[i] + " is compromised/fake");
						return result.type;
					}
					
					i++;
				}
			}
			catch(e) {
				compromisedstores.dump(e);
				return '';
			}
		
			return '';
		},
		
		showWarningMobileDesktop: function(type) {
			compromisedstores.dump("Display warning for mobile");
			
			var text = "This store has been compromised.";

			if (type == "f") { // fake store
				text = "This is a fake store.";
			}
	
			var alertsService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
			alertsService.showAlertNotification(null, "Zscaler Safe Shopping", text, false, "", null, "");
		},
		showWarningMobilePanel: function(type) {
			compromisedstores.dump("Display warning panel for mobile");
			
			var text = "This store has been compromised.\nYou may not want to enter any confidential information or credit card number on this site.";

			if (type == "f") { // fake store
				text = "This is a fake store.\nDo not enter any information on this site.";
			}
	
			var panel = document.getElementById("compromisedstores-panel");
			panel.hidden = false;
			
			setTimeout(function(panel) { panel.hidden = true; }, 5000, panel);
		},
		showWarningMobilePopup: function(type) {
			compromisedstores.dump("Display warning panel for mobile");
			
			var text = "This store has been compromised.\nYou may not want to enter any confidential information or credit card number on this site.";

			if (type == "f") { // fake store
				text = "This is a fake store.\nDo not enter any information on this site.";
			}
			alert(text);
		},
		
		showWarning: function(tab, type) {

			compromisedstores.dump("Display warning");

			var notificationBox = gBrowser.getNotificationBox(tab);
			var notification = notificationBox.currentNotification;
			if (notification == null) { // no notification currently displayed

				var text = "This store has been compromised.";
				if (type == "f") { // fake store
					text = "This is a fake store.";
				}
					notification = notificationBox.appendNotification( text , "compromisedstorese" , "chrome://compromisedstores/content/zscaler.png" , notificationBox.PRIORITY_INFO_HIGH , null);


				var messageText = document.getAnonymousElementByAttribute(notification, "anonid", "messageText");

				// Move the buttons to the left
				messageText.parentNode.setAttribute("flex", 0);
				var spacer = messageText.parentNode.removeChild(messageText.nextSibling);
				messageText.parentNode.parentNode.insertBefore(spacer, messageText.parentNode.parentNode.lastChild);

				var url = 'http://www.zscaler.com/zsafeshop/safeshoppingdetails.html#' + type;

				var fragment = document.createDocumentFragment();
	// 			fragment.appendChild(document.createTextNode("Link: "));
				var link = document.createElementNS("http://www.w3.org/1999/xhtml", "html:a");
				
				link.setAttribute('href', '#');
				link.addEventListener("click", function () { gBrowser.addTab(url);}, false);
				link.setAttribute('id', 'notification-link');
				link.setAttribute('style', 'color: Blue;');
				link.appendChild(document.createTextNode(text));
				fragment.appendChild(link);
				
				fragment.appendChild(document.createTextNode(" "));
				
				var text = document.createElementNS("http://www.w3.org/1999/xhtml", "html:b");
				if (type == "f") {
					text.appendChild(document.createTextNode("Do not enter any information on this site."));
				}
				else {
					text.appendChild(document.createTextNode("You may not want to enter any confidential information or credit card number on this site."));
				}
				fragment.appendChild(text);
				
				fragment.appendChild(document.createTextNode(" "));
				
				link = document.createElementNS("http://www.w3.org/1999/xhtml", "html:a");
				link.setAttribute('href', '#');
				link.addEventListener("click", function () { gBrowser.addTab(url);}, false);
				link.setAttribute('id', 'notification-link2');
				link.setAttribute('style', 'color: Blue;');
				link.appendChild(document.createTextNode("More info?"));
				fragment.appendChild(link);

				messageText.removeChild(messageText.firstChild);
				messageText.appendChild(fragment);

	// 			messageText.parentNode.parentNode.removeChild(messageText.parentNode.parentNode.children[2]);
	// 			messageText.parentNode.parentNode.children[2].addEventListener("click", function(e) { alert("Close!"); }, false );
			}
	// 		else {
	// 			this.dump("Notification already exists");
	// 		}
		},
		
		// Tab listener
		onLocationChange: function(aBrowser, webProgress, request, location) {
			if (compromisedstores.prefManager.getBoolPref("extensions.compromisedstores.enabled") == false || typeof(location.host) == "undefined" || location.host == null || location.host == '') {
				compromisedstores.dump("No location.hostname");
				return;
			}
			
			try {
				compromisedstores.dump("Location has changed to " + location.host);
				
				var type = compromisedstores.domainType(location.host);
				
				if (type != '') {
					compromisedstores.showWarning(aBrowser, type);
				}
			}
			catch(e) {
				compromisedstores.dump(e);
			}
		},
		onProgressChange: function(aBrowser, webProgress, request, curSelfProgress, maxSelfProgress, curTotalProgress, maxTotalProgress) { },
		onSecurityChange: function(aBrowser, aWebProgress, aRequest, aState ) { },
		onStateChange: function(aBrowser, aWebProgress, aRequest, aStateFlags, aStatus) { },
		onStatusChange: function(aBrowser, aWebProgress, aRequest, aStatus, aMessage) { },
		onRefreshAttempted: function(aBrowser, webProgress, aRefreshURI, aMillis, aSameURI) {
			return true;
		},
		onLinkIconAvailable: function(aBrowser) { },
		
		getDomains: function(host) {
			var data = host.split('.'); // get all the "subdomains"
			var domains = [];
			
			var domain = data.pop();
			while (data.length > 0) {
				domain = data.pop() + "." + domain;
				domains.push(domain);
				
				compromisedstores.dump("Added domain " + domain);
			}
			
			return domains;
		},
		
		updateStatus: function() {
			var status = "Zscaler Safe Shopping " + this.strings.getString("extensions.compromisedstores.version");
			status += "\nStatus : ";
			
			if (this.prefManager.getBoolPref("extensions.compromisedstores.enabled") == false) {
				status += "Off";
				document.getElementById("compromisedstores-status-bar-icon").setAttribute("src", "chrome://compromisedstores/content/zscaler-small-off.png");
			}
			else {
				status += "On";
				document.getElementById("compromisedstores-status-bar-icon").setAttribute("src", "chrome://compromisedstores/content/zscaler-small.png");
			}
			document.getElementById("compromisedstores-status-bar-icon").setAttribute("tooltiptext", status);
		},
		
		switchStatus: function() {
			if (this.prefManager.getBoolPref("extensions.compromisedstores.enabled") == false) {
				this.prefManager.setBoolPref("extensions.compromisedstores.enabled", true);
			}
			else {
				this.prefManager.setBoolPref("extensions.compromisedstores.enabled", false);
			}
			this.updateStatus();
		},
		
		checkUpdate: function() {
			var interval = this.prefManager.getIntPref("extensions.compromisedstores.days") * 24 * 60 * 60 * 1000;
			var last_update = this.prefManager.getIntPref("extensions.compromisedstores.last_update") * 1000;
			var currentDate = new Date();
			
			if (last_update + interval <= currentDate.getTime()) {
				this.fetchDomains();
			}
		},
		
		md5sum: function(val) {
			var bytes = { };

			var data = compromisedstores.converter.convertToByteArray(val, bytes);
			
			compromisedstores.md5.init(compromisedstores.md5.MD5);
			compromisedstores.md5.update(data, data.length);
			var hash = compromisedstores.md5.finish(false);
			
			// convert the binary hash data to a hex string.
			var hex = [compromisedstores.toHexString(hash.charCodeAt(i)) for (i in hash)].join(""); 
		
			return hex;
		}
	};

	window.addEventListener("load",  function() { compromisedstores = new compromisedstores(); compromisedstores.register(); }, false);
	window.addEventListener("unload", function() { compromisedstores.onUnload() }, false);
}
